@@ -1,4 +1,4 @@
-# $Id: S2k.pm,v 1.5 2001/07/27 19:39:32 btrott Exp $
+# $Id: S2k.pm,v 1.8 2001/07/29 06:07:17 btrott Exp $
 
 package Crypt::OpenPGP::S2k;
 use strict;
@@ -18,16 +18,19 @@ use vars qw( %TYPES );
 sub new {
     my $class = shift;
     my $type = shift;
-    my $buffer = shift;
     $type = $TYPES{ $type } || $type;
-    if (!$type && $buffer) {
-        my $id = $buffer->get_int8;
-        $type = $TYPES{$id};
-    }
     return $class->error("Invalid type of S2k") unless $type;
     my $pkg = join '::', __PACKAGE__, $type;
     my $s2k = bless { }, $pkg;
-    $s2k->init($buffer);
+    $s2k->init(@_);
+}
+
+sub parse {
+    my $class = shift;
+    my($buf) = @_;
+    my $id = $buf->get_int8;
+    my $type = $TYPES{$id};
+    $class->new($type, $buf);
 }
 
 sub init { $_[0] }
@@ -72,6 +75,14 @@ sub init {
 
 sub s2k { $_[0]->{hash}->hash($_[2] . $_[1]) }
 
+sub save {
+    my $s2k = shift;
+    my $buf = Crypt::OpenPGP::Buffer->new;
+    $buf->put_int8(1);
+    $buf->put_int8($s2k->{hash_alg});
+    $buf->bytes;
+}
+
 package Crypt::OpenPGP::S2k::Salted;
 use base qw( Crypt::OpenPGP::S2k );
 
@@ -97,6 +108,15 @@ sub init {
 
 sub s2k { $_[0]->{hash}->hash($_[2] . $_[1] . $_[0]->{salt}) }
 
+sub save {
+    my $s2k = shift;
+    my $buf = Crypt::OpenPGP::Buffer->new;
+    $buf->put_int8(2);
+    $buf->put_int8($s2k->{hash_alg});
+    $buf->put_bytes($s2k->{salt});
+    $buf->bytes;
+}
+
 package Crypt::OpenPGP::S2k::Salt_Iter;
 use base qw( Crypt::OpenPGP::S2k );
 
@@ -153,7 +173,69 @@ sub save {
     $buf->put_int8($s2k->{hash_alg});
     $buf->put_bytes($s2k->{salt});
     $buf->put_int8($s2k->{count});
-    $buf;
+    $buf->bytes;
 }
 
 1;
+__END__
+
+=head1 NAME
+
+Crypt::OpenPGP::S2k - String-to-key generation
+
+=head1 SYNOPSIS
+
+    use Crypt::OpenPGP::S2k;
+
+    my $s2k = Crypt::OpenPGP::S2k->new('Salt_Iter');
+    my $key = $s2k->generate($passphrase, $keysize);
+
+    my $serialized = $s2k->save;
+
+=head1 DESCRIPTION
+
+I<Crypt::OpenPGP::S2k> implements string-to-key generation for use in
+generating symmetric cipher keys from standard, arbitrary-length
+passphrases (like those used to lock secret key files). Since a
+passphrase can be of any length, and key material must be a very
+specific length, a method is needed to translate the passphrase into
+the key. The OpenPGP RFC defines three such methods, each of which
+this class implements.
+
+=head1 USAGE
+
+=head2 Crypt::OpenPGP::S2k->new($type)
+
+Creates a new type of S2k-generator of type I<$type>; valid values for
+I<$type> are C<Simple>, C<Salted>, and C<Salt_Iter>. These generator
+types are described in the OpenPGP RFC section 3.6.
+
+Returns the new S2k-generator object.
+
+=head2 Crypt::OpenPGP::S2k->parse($buffer)
+
+Given a buffer I<$buffer> of type I<Crypt::OpenPGP::Buffer>, determines
+the type of S2k from the first octet in the buffer (one of the types
+listed above in I<new>), then creates a new object of that type and
+initializes the S2k state from the buffer I<$buffer>. Different
+initializations occur based on the type of S2k.
+
+Returns the new S2k-generator object.
+
+=head2 $s2k->save
+
+Serializes the S2k object and returns the serialized form; this form
+will differ based on the type of S2k.
+
+=head2 $s2k->generate($passphrase, $keysize)
+
+Given a passphrase I<$passphrase>, which should be a string of octets
+of arbitrary length, and a keysize I<$keysize>, generates enough key
+material to meet the size I<$keysize>, and returns that key material.
+
+=head1 AUTHOR & COPYRIGHTS
+
+Please see the Crypt::OpenPGP manpage for author, copyright, and
+license information.
+
+=cut
